=begin
  Copyright 2014 Maxine Red <maxine@furfind.net>

  This file is part of toy-chooser.

  toy-chooser is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  toy-chooser is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with toy-chooser.  If not, see <http://www.gnu.org/licenses/>.
=end

require "sqlite3"
require "net/http"
require "json"
require "nokogiri"

require "manufacturer/bad-dragon"

class Chooser
  def initialize
    @db_file = File.expand_path("~/.toys.sqlite")
    if !File.exist?(@db_file) then
      database_create
    end
  end
  # Update database for given manufacturer.
  def update(manu)
    case manu.first
    when "bad-dragon" then fetch_bd
    else
      $stderr.puts "Manufacturer unkown.","Known manufacturers are: bad-dragon"
      abort
    end
  end
  # Add a toy to the 'owned' list. Toys are in the format sku/size.
  def add(toys)
    toys.each do |toy|
      sku, size = toy.split("/")
      sku, size = SQLite3::Database.quote(sku), SQLite3::Database.quote(size.upcase)
      SQLite3::Database.new(@db_file) do |db|
        query   = "SELECT id, name FROM toys WHERE sku = '#{sku}'"
        query  += " AND size = '#{size}';"
        id, name = db.execute(query).flatten
        if !id then
          $stderr.puts "Unknown toy or size \"#{toy}\"."
          next
        end
        query  = "INSERT INTO owned (sku, size, toy_id) VALUES"
        query += " ('#{sku}','#{size}',#{id});"
        db.execute(query)
        puts "Added \"#{name}\" (#{size}) to owned list."
      end
    end
  end
  # Show measurements of toys.
  def show(toys)
    toys.each do |toy|
      sku, size = toy.split("/")
      sku, size = SQLite3::Database.quote(sku), SQLite3::Database.quote(size.to_s.upcase)
      SQLite3::Database.new(@db_file) do |db|
        query  = "SELECT sku, size, price, dia_head, dia_shaft, dia_knot, "
        query += "circ_head, circ_shaft, circ_knot, usable_length, "
        query += "total_length FROM toys WHERE sku = '#{sku}';"
        draw_show(db.execute(query))
      end
    end
  end
  # Suggests toys for future purchases guessed from what is owned already.
  def suggest(noargs=[])
    i, sizes = 0, Array.new(7) {0}
    ranges = [0.5, 0.4, 0.3, 0.6, 0.5, 0.4, 0.9]
    cols = ["dia_head","dia_shaft","dia_knot","circ_head","circ_shaft",
            "circ_knot","usable_length"]
    SQLite3::Database.new(@db_file) do |db|
      query  = "SELECT #{cols.join(", ")} FROM toys, owned "
      query += "WHERE toys.id = owned.toy_id;"
      db.execute(query) do |row|
        row.each_with_index do |c,id|
          sizes[id] += c.to_i
        end
        i += 1
      end
      sizes = sizes.map.with_index do |x,id|
        avg = (x/i.to_f).round.to_i
        line  = "((#{cols[id]} >= #{(avg*(1-ranges[id])).round.to_i} "
        line += "AND #{cols[id]} <= #{(avg*(1+ranges[id])).round.to_i}) "
        line += "OR #{cols[id]} IS NULL)"
        line
      end
      query  = "SELECT sku, size FROM toys WHERE #{sizes.join(" AND ")};"
      draw_list(db.execute(query))
    end
  end
  # List existing toys.
  def list(toy_list="toys")
    toy_list = toy_list.first
    toy_list = "toys" unless toy_list
    if !["toys","owned"].include?(toy_list) then
      $stderr.puts "No such list."
      abort
    end
    SQLite3::Database.new(@db_file) do |db|
      draw_list(db.execute("SELECT sku, size FROM #{toy_list};"))
    end
  end
  private
  def draw_list(sql_list)
    toys = Hash.new
    sizes = ["XS","S ","M ","L ","XL","O "]
    deli = "+-"+"-"*15+"-+"+"----+"*sizes.count+""+"\n"
    print deli
    puts "| #{" "*15} | #{sizes.join(" | ")} |"
    print deli
    sql_list.each do |t|
      if !toys.has_key?(t[0]) then
        toys.store(t[0],[t[1]])
      else
        toys[t[0]] << t[1]
      end
    end
    toys.each do |k,v|
      cols = sizes.map do |c|
        v.include?(c.sub(" ","")) ? c : "  "
      end
      puts "| #{k.pad(15)} | #{cols.join(" | ")} |"
    end
    print deli
  end
  def draw_show(sql_list)
    toys = Hash.new
    sizes = ["  XS   ","   S   ","   M   ","   L   ","  XL   ","   O   "]
    measurements = ["Price", "Dia. of head", "Dia. of shaft",
                    "Dia. of speccial", "Circ. of head",
                    "Circ. of shaft", "Circ. of special",
                    "Usable length", "Total length"]
    deli = "+-"+"-"*15+"-+"+"---------+"*sizes.count+""+"\n"
    print deli
    puts "| #{sql_list.first.first.pad(15)} | #{sizes.join(" | ")} |"
    print deli
    sql_list.map{|c|c.shift;c}.each do |t|
      toys.store(t.shift,t)
    end
    measurements.each_with_index do |m,id|
      cols = sizes.map do |c|
        c.gsub!(" ","")
        toys[c] ? toys[c][id].to_i : 0
      end
      if m == "Price" then
        cols.map!{|c|c.pad(4," ").sub(/\s(\d)/,"$\\1")+".00"}
      else
        cols.map!{|c|(c/100.0).pad(7," ")}
      end
      puts "| #{m.pad(15)} | #{cols.join(" | ")} |"
      print deli if m == "Price"
    end
    print deli
  end
  def database_create
    SQLite3::Database.new(@db_file) do |db|
      db.execute("CREATE TABLE toys (
                 id INTEGER PRIMARY KEY,
                 sku VARCHAR(127),
                 name VARCHAR(255),
                 size VARCHAR(3),
                 price INTEGER,
                 dia_head INTEGER,
                 dia_shaft INTEGER,
                 dia_knot INTEGER,
                 circ_head INTEGER,
                 circ_shaft INTEGER,
                 circ_knot INTEGER,
                 usable_length INTEGER,
                 total_length INTEGER
                );")
      db.execute("CREATE TABLE owned (
                 id INTEGER PRIMARY KEY,
                 sku VARCHAR(127),
                 size VARCHAR(3),
                 toy_id INTEGER REFERENCES toys(id)
                 );")
    end
  end
end
